require "hashie"

# This is a utility class that makes it easy to grab deeply nested values safely. Given a Hash like:
# fetcher = DataFetcher.new(
#   {
#     "names"=>[{
#       "type"=>{"code"=>"PRI", "description"=>"Primary"},
#       "familyName"=>"Montalban",
#       "givenName"=>"Ricardo",
#     }],
#   }
#  )
#
# you could access the name code like this:
#   fetcher.names.first.type.code.value
#
# If any part of the path is missing or nil, no exception will be raised, and the value() call
# will return nil.
#
# By default, you'll get nil back if any part of call chain is missing or nil. If you want a different
# fallback value, you can pass that to value(). This would return "not found":
#   fetcher.thisIsNotInTheResponseHash.first.type.code.value("not found")
#
# DataFetcher will accept node names in camel case or snake case. The following are equivalent:
#   fetcher.names.first.familyName.value
#   fetcher.names.first.family_name.value
#

module Ucb
  module Hcm
    class DataFetcher
      attr_reader :current_value

      def initialize(raw_data)
        @original_value = raw_data
        reset_current_value(raw_data)
      end

      def value(default = nil)
        (@current_value || default).tap do
          reset_current_value(@original_value)
        end
      end

      # This is where the fetching logic happens - each time we're given a value to find, we store
      # that value as @current_value and wait for the next call. If we ever get nil, then none of the
      # subsequent calls matter, so we short-circuit them.
      def method_missing(method, *args)
        return self if @current_value.nil?

        normalized_method = camelize(method)
        if @current_value.respond_to?(normalized_method)
          @current_value = @current_value.send(normalized_method, *args)
        else
          @current_value = nil
        end
        self
      end

      private

      def camelize(str)
        str.to_s.gsub(/_([a-z])/) { "#{$1.upcase}" }
      end

      def reset_current_value(hash)
        @current_value = Hashie::Mash.new(hash)
      end

    end
  end
end
